#!/usr/bin/env python

import time
import wx
from wx.lib.wordwrap import wordwrap

class CDown(wx.Frame):
    """Tiny alarm clock which flashes and plays a sound when done"""
    def __init__(self, parent, id):
        wx.Frame.__init__(self, parent, id, title='Timer Alarm', size=(300,150))

        self.sound = wx.Sound('horn.wav')
        self.flashing = False
        self.running = False
        self.th = 0
        self.tm = 0
        self.ts = 0
        self.dt = 500 # timer interval in ms

        self.txth = wx.StaticText(self, -1, "Hours")
        self.txtm = wx.StaticText(self, -1, "Minutes")
        self.txts = wx.StaticText(self, -1, "Seconds")

        self.ledh = wx.SpinCtrl(self, -1, "", min=0, max=99, initial=0)
        self.ledm = wx.SpinCtrl(self, -1, "", min=0, max=99, initial=0)
        self.leds = wx.SpinCtrl(self, -1, "", min=0, max=99, initial=0)

        self.startbtn = wx.Button(self, -1, "Start")
        self.optionsbtn = wx.Button(self, -1, "Options")
        self.resetbtn = wx.Button(self, -1, "Reset")
        self.aboutbtn = wx.Button(self, wx.ID_ABOUT, "About")

        self.optionsbtn.Enable(0)

        timer_id = wx.NewId()
        donetimer_id = wx.NewId()
        self.timer = wx.Timer(self, timer_id)
        self.donetimer = wx.Timer(self, donetimer_id)
        self.Bind(wx.EVT_TIMER, self.OnTimer, self.timer)
        self.Bind(wx.EVT_TIMER, self.OnDoneTimer, self.donetimer)
        self.Bind(wx.EVT_BUTTON, self.OnStart, self.startbtn)
        self.Bind(wx.EVT_BUTTON, self.OnReset, self.resetbtn)
        self.Bind(wx.EVT_BUTTON, self.OnOptions, self.optionsbtn)
        self.Bind(wx.EVT_BUTTON, self.OnAbout, self.aboutbtn)
        self.Bind(wx.EVT_SPINCTRL, self.OnSpinCtrlh, self.ledh)
        self.Bind(wx.EVT_SPINCTRL, self.OnSpinCtrlm, self.ledm)
        self.Bind(wx.EVT_SPINCTRL, self.OnSpinCtrls, self.leds)
        #self.Bind(wx.EVT_TEXT, self.OnSpinCtrlh, self.ledh)
        #self.Bind(wx.EVT_TEXT, self.OnSpinCtrlm, self.ledm)
        #self.Bind(wx.EVT_TEXT, self.OnSpinCtrls, self.leds)
        self.Bind(wx.EVT_CLOSE, self.OnClose)

        sizer = wx.GridBagSizer(hgap=5, vgap=5)
        sizer.Add(self.txth, pos=(0,0), flag=wx.ALIGN_CENTER)
        sizer.Add(self.txtm, pos=(0,1), flag=wx.ALIGN_CENTER)
        sizer.Add(self.txts, pos=(0,2), flag=wx.ALIGN_CENTER)
        sizer.Add(self.ledh, pos=(1,0))
        sizer.Add(self.ledm, pos=(1,1))
        sizer.Add(self.leds, pos=(1,2))
        sizer.Add(self.startbtn, pos=(2,0), span=(2,1), flag=wx.EXPAND)
        sizer.Add(self.resetbtn, (2,1))
        sizer.Add(self.aboutbtn, (3,2))
        sizer.Add(self.optionsbtn, (2,2))

        mainsizer = wx.BoxSizer(wx.VERTICAL)
        mainsizer.Add(sizer, -1, wx.EXPAND|wx.ALL, 10)
        self.SetSizerAndFit(mainsizer)
        self.Centre()

    def OnSpinCtrlh(self, event):
        self.th = self.ledh.GetValue()
        if self.ledh.GetValue() < 0:
            self.th = 0
            self.ledh.SetValue(0)
        self.GetCDown()

    def OnSpinCtrlm(self, event):
        #self.tm = self.ledm.GetValue()
        #if self.tm < 0:
            #self.tm = 0
            #self.ledm.SetValue(0)
        #self.GetCDown()
        self.tm = self.ledm.GetValue()
        self.countdown = 3600*self.th + 60*self.tm + self.ts

    def OnSpinCtrls(self, event):
        self.ts = self.leds.GetValue()
        if self.leds.GetValue() < 0:
            self.ts = 0
            self.leds.SetValue(0)
        self.GetCDown()

    def OnTimer(self, event):
        """Update times"""
        te = int(time.time()) - self.starttime #elapsed time
        t = self.countdown - te #remaining time
        self.th = t/3600
        self.tm = (t%3600)/60
        self.ts = (t%3600)%60
        self.ledh.SetValue(t/3600)
        self.ledm.SetValue((t%3600)/60)
        self.leds.SetValue((t%3600)%60)
        if (t <= 0):
            self.TimerDone()

    def OnStart(self, event):
        if not self.running:
            self.ts = self.leds.GetValue()
            self.tm = self.ledm.GetValue()
            self.th = self.ledh.GetValue()
            self.countdown = 3600*self.th + 60*self.tm + self.ts
            if self.countdown != 0:
                self.running = True
                self.startbtn.SetLabel('Stop')
                self.starttime = int(time.time())
                self.timer.Start(1000)
        else:
            self.running = False
            self.startbtn.SetLabel('Start')
            if self.flashing:
                self.sound.Stop()
                self.donetimer.Stop()
                self.flashing = 0
                self.startbtn.SetBackgroundColour(self.defbckgcol)
            else:
                self.timer.Stop()
                self.GetCDown()

    def OnClose(self, event):
        self.timer.Stop()
        self.donetimer.Stop()
        frame.Destroy()

    def TimerDone(self):
        self.timer.Stop()
        self.defbckgcol = self.startbtn.GetBackgroundColour()
        self.flashing = 1
        self.donetimer.Start(200)
        self.sound.Play(wx.SOUND_ASYNC|wx.SOUND_LOOP)
        self.Raise()

    def OnDoneTimer(self, event):
        if self.startbtn.GetBackgroundColour() == self.defbckgcol:
            self.startbtn.SetBackgroundColour('RED')
        else:
            self.startbtn.SetBackgroundColour(self.defbckgcol)

    def GetCDown(self):
        #self.countdown = 3600*self.th + 60*self.tm + self.ts
        self.ts = self.leds.GetValue()
        self.tm = self.ledm.GetValue()
        self.th = self.ledh.GetValue()
        self.countdown = 3600*self.th + 60*self.tm + self.ts

    def OnOptions(self, event):
        # No options yet
        pass

    def OnReset(self, event):
        self.timer.Stop()
        self.donetimer.Stop()
        self.ts = 0
        self.tm = 0
        self.th = 0
        self.ledh.SetValue(0)
        self.ledm.SetValue(0)
        self.leds.SetValue(0)
        self.flashing = 1
        self.startbtn.SetLabel('Start')
        self.startbtn.SetBackgroundColour(self.defbckgcol)

    def OnAbout(self, event):
        info = wx.AboutDialogInfo()
        info.Name = "Timer Alarm"
        #info.Version = "1.0"
        info.Copyright = "(C) 2011 K. Ilker Sen"

        #info.SetIcon(wx.Icon('icons/hunter.png', wx.BITMAP_TYPE_PNG))

        info.Description = wordwrap("Timer Alarm is a simple, tiny, and useful "
        "countdown timer application written in pure python.",
            350, wx.ClientDC(self))

        info.WebSite = 'http://ilkersen.net'
        info.Licence= wordwrap("This program is free software: you can redistribute "
            "it and/or modify it under the terms of the GNU General Public License "
            "as published by the Free Software Foundation, either version 3 of the "
            "License, or any later version.\n\n"
            "This program is distributed in the hope that it will be useful,"
            "but WITHOUT ANY WARRANTY; without even the implied warranty of"
            "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the"
            "GNU General Public License for more details.\n\n"
            "You should have received a copy of the GNU General Public License"
            "along with this program.  If not, see <http://www.gnu.org/licenses/>.",
            500, wx.ClientDC(self))
        wx.AboutBox(info)


if __name__ == '__main__':
    app = wx.App()
    frame = CDown(None, -1)
    frame.Show(True)
    app.SetTopWindow(frame)
    app.MainLoop()
